package br.ufes.inf.decisionelement;

import br.ufes.inf.decisionelement.networkprofile.configuration.NetworkConfiguration;
import br.ufes.inf.decisionelement.networkprofile.policy.NetworkPolicy;
import br.ufes.inf.decisionelement.networkprofile.goal.NetworkGoal;
import java.util.ArrayList;
import org.onosproject.cfg.ComponentConfigService;
import org.onosproject.core.CoreService;
import org.onosproject.net.topology.TopologyService;
import org.onosproject.net.flowobjective.FlowObjectiveService;
import org.onosproject.net.flow.FlowRuleService;
import org.onosproject.net.host.HostService;
import org.onosproject.net.intent.IntentService;
import org.onosproject.net.packet.PacketService;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Modified;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.Dictionary;
import java.util.List;
import java.util.Properties;

import static org.onlab.util.Tools.get;

/**
 * ONOS application component.
 */
@Component(
        immediate = true,
        service = { AppComponentInterface.class },
        property = {
            "networkGoals=[]",
            "networkPolicies=[]",
            "networkConfiguration=[]",
        }
)
public class AppComponent implements AppComponentInterface {
    // Configurable properties that will show up in ONOS's settings.
    private String networkGoals;
    private String networkPolicies;
    private String networkConfiguration;

    private boolean isActive = false;

    private List<NetworkGoal> networkGoalEntities = new ArrayList<>();
    private List<NetworkPolicy> networkPolicyEntities = new ArrayList<>();
    private List<NetworkConfiguration> networkConfigurationEntities = new ArrayList<>();

    private final Logger log = LoggerFactory.getLogger(getClass());

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected CoreService coreService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected FlowObjectiveService flowObjectiveService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected FlowRuleService flowRuleService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected HostService hostService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected IntentService intentService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected PacketService packetService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected TopologyService topologyService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    protected ComponentConfigService cfgService;

    @Activate
    protected void activate() {
        cfgService.registerProperties(getClass());
        this.isActive = true;
        log.info("Started " + this.getClass().getCanonicalName());
        this.parseConfiguration();
    }

    @Deactivate
    protected void deactivate() {
        cfgService.unregisterProperties(getClass(), false);
        this.isActive = false;
        log.info("Stopped " + this.getClass().getCanonicalName());
    }

    @Modified
    public void modified(ComponentContext context) {
        Dictionary<?, ?> properties = context != null ? context.getProperties() : new Properties();
        if (context != null) {
            this.networkGoals = get(properties, "networkGoals");
            this.networkPolicies = get(properties, "networkPolicies");
            this.networkConfiguration = get(properties, "networkConfiguration");
        }
        if (this.networkGoals == null) {
            this.networkGoals = "[]";
        }
        if (this.networkPolicies == null) {
            this.networkPolicies = "[]";
        }
        if (this.networkConfiguration == null) {
            this.networkConfiguration = "[]";
        }
        log.info("Reconfigured " + this.getClass().getCanonicalName());
        this.parseConfiguration();
    }

    private void parseConfiguration() {
        if (this.isActive) {
            System.out.print("");
        }
    }

    @Override
    public void someMethod() {
        log.info("Invoked " + this.getClass().getCanonicalName());
    }

}
